@@ -24,6 +24,9 @@ parse_model <- function(model, backend, ...) {
 .parse_model_cmdstanr <- function(model, silent = 1, ...) {
   require_package("cmdstanr")
   temp_file <- cmdstanr::write_stan_file(model)
+  if (cmdstanr::cmdstan_version() >= "2.29.0") {
+    .canonicalize_stan_model(temp_file, overwrite_file = TRUE)
+  }
   out <- eval_silent(
     cmdstanr::cmdstan_model(temp_file, compile = FALSE, ...),
     type = "message", try = TRUE, silent = silent
@@ -94,6 +97,9 @@ compile_model <- function(model, backend, ...) {
   require_package("cmdstanr")
   args <- list(...)
   args$stan_file <- cmdstanr::write_stan_file(model)
+  if (cmdstanr::cmdstan_version() >= "2.29.0") {
+    .canonicalize_stan_model(args$stan_file, overwrite_file = TRUE)
+  }
   if (use_threading(threads)) {
     args$cpp_options$stan_threads <- TRUE
   }
@@ -136,10 +142,10 @@ fit_model <- function(model, backend, ...) {
 # @param model a compiled Stan model
 # @param sdata named list to be passed to Stan as data
 # @return a fitted Stan model
-.fit_model_rstan <- function(model, sdata, algorithm, iter, warmup, thin, 
-                             chains, cores, threads, opencl, inits, exclude, 
+.fit_model_rstan <- function(model, sdata, algorithm, iter, warmup, thin,
+                             chains, cores, threads, opencl, init, exclude,
                              seed, control, silent, future, ...) {
-  
+
   # some input checks and housekeeping
   if (use_threading(threads)) {
     if (utils::packageVersion("rstan") >= 2.26) {
@@ -155,19 +161,21 @@ fit_model <- function(model, backend, ...) {
     stop2("OpenCL is not supported by backend 'rstan' version ",
           utils::packageVersion("rstan"), ".")
   }
-  if (is.character(inits) && !inits %in% c("random", "0")) {
-    inits <- get(inits, mode = "function", envir = parent.frame())
+  if (is.null(init)) {
+    init <- "random"
+  } else if (is.character(init) && !init %in% c("random", "0")) {
+    init <- get(init, mode = "function", envir = parent.frame())
   }
   args <- nlist(
-    object = model, data = sdata, iter, seed, 
-    init = inits, pars = exclude, include = FALSE
+    object = model, data = sdata, iter, seed,
+    init = init, pars = exclude, include = FALSE
   )
   dots <- list(...)
   args[names(dots)] <- dots
-  
+
   # do the actual sampling
   if (silent < 2) {
-    message("Start sampling") 
+    message("Start sampling")
   }
   if (algorithm %in% c("sampling", "fixed_param")) {
     c(args) <- nlist(warmup, thin, control, show_messages = !silent)
@@ -182,23 +190,23 @@ fit_model <- function(model, backend, ...) {
       futures <- fits <- vector("list", chains)
       for (i in seq_len(chains)) {
         args$chain_id <- i
-        if (is.list(inits)) {
-          args$init <- inits[i]
+        if (is.list(init)) {
+          args$init <- init[i]
         }
         futures[[i]] <- future::future(
-          brms::do_call(rstan::sampling, args), 
+          brms::do_call(rstan::sampling, args),
           packages = "rstan",
           seed = TRUE
         )
       }
       for (i in seq_len(chains)) {
-        fits[[i]] <- future::value(futures[[i]]) 
+        fits[[i]] <- future::value(futures[[i]])
       }
       out <- rstan::sflist2stanfit(fits)
       rm(futures, fits)
     } else {
       c(args) <- nlist(chains, cores)
-      out <- do_call(rstan::sampling, args) 
+      out <- do_call(rstan::sampling, args)
     }
   } else if (algorithm %in% c("fullrank", "meanfield")) {
     # vb does not support parallel execution
@@ -215,27 +223,31 @@ fit_model <- function(model, backend, ...) {
 # @param model a compiled Stan model
 # @param sdata named list to be passed to Stan as data
 # @return a fitted Stan model
-.fit_model_cmdstanr <- function(model, sdata, algorithm, iter, warmup, thin, 
-                                chains, cores, threads, opencl, inits, exclude, 
+.fit_model_cmdstanr <- function(model, sdata, algorithm, iter, warmup, thin,
+                                chains, cores, threads, opencl, init, exclude,
                                 seed, control, silent, future, ...) {
-  
+
   require_package("cmdstanr")
   # some input checks and housekeeping
   class(sdata) <- "list"
   if (isNA(seed)) {
     seed <- NULL
   }
-  if (is_equal(inits, "random")) {
-    inits <- NULL
-  } else if (is_equal(inits, "0")) {
-    inits <- 0
+  if (is_equal(init, "random")) {
+    init <- NULL
+  } else if (is_equal(init, "0")) {
+    init <- 0
   }
   if (future) {
     stop2("Argument 'future' is not supported by backend 'cmdstanr'.")
   }
-  args <- nlist(data = sdata, seed, init = inits)
+  args <- nlist(data = sdata, seed, init)
   if (use_threading(threads)) {
-    args$threads_per_chain <- threads$threads
+    if (algorithm %in% c("sampling", "fixed_param")) {
+      args$threads_per_chain <- threads$threads
+    } else if (algorithm %in% c("fullrank", "meanfield")) {
+      args$threads <- threads$threads
+    }
   }
   if (use_opencl(opencl)) {
     args$opencl_ids <- opencl$ids
@@ -244,7 +256,7 @@ fit_model <- function(model, backend, ...) {
   dots <- list(...)
   args[names(dots)] <- dots
   args[names(control)] <- control
-  
+
   chains <- as_one_numeric(chains)
   empty_model <- chains <= 0
   if (empty_model) {
@@ -256,16 +268,16 @@ fit_model <- function(model, backend, ...) {
     thin <- 1
     cores <- 1
   }
-  
+
   # do the actual sampling
   if (silent < 2) {
-    message("Start sampling") 
+    message("Start sampling")
   }
   if (algorithm %in% c("sampling", "fixed_param")) {
     c(args) <- nlist(
       iter_sampling = iter - warmup,
-      iter_warmup = warmup, 
-      chains, thin, 
+      iter_warmup = warmup,
+      chains, thin,
       parallel_chains = cores,
       show_messages = !silent,
       fixed_param = algorithm == "fixed_param"
@@ -296,8 +308,8 @@ fit_model <- function(model, backend, ...) {
 }
 
 # fit model with a mock backend for testing
-.fit_model_mock <- function(model, sdata, algorithm, iter, warmup, thin, 
-                            chains, cores, threads, opencl, inits, exclude, 
+.fit_model_mock <- function(model, sdata, algorithm, iter, warmup, thin,
+                            chains, cores, threads, opencl, init, exclude,
                             seed, control, silent, future, mock_fit, ...) {
   if (is.function(mock_fit)) {
     out <- mock_fit()
@@ -331,7 +343,7 @@ needs_recompilation <- function(x) {
     out <- FALSE
   } else if (backend == "cmdstanr") {
     exe_file <- attributes(x$fit)$CmdStanModel$exe_file()
-    out <- !is.character(exe_file) || !exists(exe_file)
+    out <- !is.character(exe_file) || !file.exists(exe_file)
   } else if (backend == "mock") {
     out <- FALSE
   }
@@ -339,20 +351,20 @@ needs_recompilation <- function(x) {
 }
 
 #' Recompile Stan models in \code{brmsfit} objects
-#' 
+#'
 #' Recompile the Stan model inside a \code{brmsfit} object, if necessary.
 #' This does not change the model, it simply recreates the executable
-#' so that sampling is possible again. 
-#' 
+#' so that sampling is possible again.
+#'
 #' @param x An object of class \code{brmsfit}.
 #' @param recompile Logical, indicating whether the Stan model should be
 #'   recompiled. If \code{NULL} (the default), \code{recompile_model} tries
 #'   to figure out internally, if recompilation is necessary. Setting it to
 #'   \code{FALSE} will cause \code{recompile_model} to always return the
 #'   \code{brmsfit} object unchanged.
-#'   
+#'
 #' @return A (possibly updated) \code{brmsfit} object.
-#' 
+#'
 #' @export
 recompile_model <- function(x, recompile = NULL) {
   stopifnot(is.brmsfit(x))
@@ -366,7 +378,7 @@ recompile_model <- function(x, recompile = NULL) {
   message("Recompiling the Stan model")
   backend <- x$backend %||% "rstan"
   new_model <- compile_model(
-    stancode(x), backend = backend, threads = x$threads, 
+    stancode(x), backend = backend, threads = x$threads,
     opencl = x$opencl, silent = 2
   )
   if (backend == "rstan") {
@@ -392,7 +404,7 @@ elapsed_time <- function(x) {
       sampling = out[, "sample"]
     )
     out$total <- out$warmup + out$sampling
-    rownames(out) <- NULL 
+    rownames(out) <- NULL
   } else if (backend == "cmdstanr") {
     out <- attributes(x$fit)$metadata$time$chains
   } else if (backend == "mock") {
@@ -422,12 +434,12 @@ require_backend <- function(backend, x) {
 }
 
 #' Threading in Stan
-#' 
+#'
 #' Use threads for within-chain parallelization in \pkg{Stan} via the \pkg{brms}
 #' interface. Within-chain parallelization is experimental! We recommend its use
 #' only if you are experienced with Stan's \code{reduce_sum} function and have a
 #' slow running model that cannot be sped up by any other means.
-#' 
+#'
 #' @param threads Number of threads to use in within-chain parallelization.
 #' @param grainsize Number of observations evaluated together in one chunk on
 #'   one of the CPUs used for threading. If \code{NULL} (the default),
@@ -438,7 +450,7 @@ require_backend <- function(backend, x) {
 #'   \code{reduce_sum}? Defaults to \code{FALSE}. Setting it to \code{TRUE}
 #'   is required to achieve exact reproducibility of the model results
 #'   (if the random seed is set as well).
-#' 
+#'
 #' @return A \code{brmsthreads} object which can be passed to the
 #'   \code{threads} argument of \code{brm} and related functions.
 #'
@@ -451,10 +463,10 @@ require_backend <- function(backend, x) {
 #'   roughly the same amount of computing time, we recommend storing
 #'   observations in random order in the data. At least, please avoid sorting
 #'   observations after the response values. This is because the latter often
-#'   cause variations in the computing time of the pointwise log-likelihood, 
+#'   cause variations in the computing time of the pointwise log-likelihood,
 #'   which makes up a big part of the parallelized code.
-#'   
-#' @examples 
+#'
+#' @examples
 #' \dontrun{
 #' # this model just serves as an illustration
 #' # threading may not actually speed things up here
@@ -464,7 +476,7 @@ require_backend <- function(backend, x) {
 #'            backend = "cmdstanr")
 #' summary(fit)
 #' }
-#' 
+#'
 #' @export
 threading <- function(threads = NULL, grainsize = NULL, static = FALSE) {
   out <- list(threads = NULL, grainsize = NULL)
@@ -499,7 +511,7 @@ validate_threads <- function(threads) {
     threads <- as_one_numeric(threads)
     threads <- threading(threads)
   } else if (!is.brmsthreads(threads)) {
-    stop2("Argument 'threads' needs to be numeric or ", 
+    stop2("Argument 'threads' needs to be numeric or ",
           "specified via the 'threading' function.")
   }
   threads
@@ -511,23 +523,23 @@ use_threading <- function(threads) {
 }
 
 #' GPU support in Stan via OpenCL
-#' 
-#' Use OpenCL for GPU support in \pkg{Stan} via the \pkg{brms} interface. Only 
+#'
+#' Use OpenCL for GPU support in \pkg{Stan} via the \pkg{brms} interface. Only
 #' some \pkg{Stan} functions can be run on a GPU at this point and so
 #' a lot of \pkg{brms} models won't benefit from OpenCL for now.
-#' 
+#'
 #' @param ids (integer vector of length 2) The platform and device IDs of the
 #'   OpenCL device to use for fitting. If you don't know the IDs of your OpenCL
 #'   device, \code{c(0,0)} is most likely what you need.
-#' 
+#'
 #' @return A \code{brmsopencl} object which can be passed to the
 #'   \code{opencl} argument of \code{brm} and related functions.
-#'   
+#'
 #' @details For more details on OpenCL in \pkg{Stan}, check out
 #' \url{https://mc-stan.org/docs/2_26/cmdstan-guide/parallelization.html#opencl}
 #' as well as \url{https://mc-stan.org/docs/2_26/stan-users-guide/opencl.html}.
-#' 
-#' @examples 
+#'
+#' @examples
 #' \dontrun{
 #' # this model just serves as an illustration
 #' # OpenCL may not actually speed things up here
@@ -537,7 +549,7 @@ use_threading <- function(threads) {
 #'            backend = "cmdstanr")
 #' summary(fit)
 #' }
-#' 
+#'
 #' @export
 opencl <- function(ids = NULL) {
   out <- list(ids = NULL)
@@ -563,7 +575,7 @@ validate_opencl <- function(opencl) {
   } else if (is.numeric(opencl)) {
     opencl <- opencl(opencl)
   } else if (!is.brmsopencl(opencl)) {
-    stop2("Argument 'opencl' needs to an integer vector or ", 
+    stop2("Argument 'opencl' needs to an integer vector or ",
           "specified via the 'opencl' function.")
   }
   opencl
@@ -606,3 +618,34 @@ repair_stanfit_names <- function(x) {
 file_refit_options <- function() {
   c("never", "always", "on_change")
 }
+
+.canonicalize_stan_model <- function(stan_file, overwrite_file = TRUE) {
+  if (os_is_windows()) {
+    stanc_cmd <- "bin/stanc.exe"
+  } else {
+    stanc_cmd <- "bin/stanc"
+  }
+  stanc_flags <- c(
+    "--auto-format",
+    "--canonicalize=deprecations,braces,parentheses"
+  )
+  if (cmdstanr::cmdstan_version() >= "2.29.0") {
+    require_package("processx")
+    res <- processx::run(
+      command = stanc_cmd,
+      args = c(stan_file, stanc_flags),
+      wd = cmdstanr::cmdstan_path(),
+      echo = FALSE,
+      echo_cmd = FALSE,
+      spinner = FALSE,
+      stderr_callback = function(x, p) {
+        message(x)
+      },
+      error_on_status = TRUE
+    )
+    if (overwrite_file) {
+      cat(res$stdout, file = stan_file, sep = "\n")
+    }
+  }
+  res$stdout
+}
