@@ -1,11 +1,15 @@
 #' Additional Response Information
-#' 
-#' Provide additional information on the response variable 
+#'
+#' Provide additional information on the response variable
 #' in \pkg{brms} models, such as censoring, truncation, or
-#' known measurement error.
-#' 
+#' known measurement error. Detailed documentation on the use
+#' of each of these functions can be found in the Details section 
+#' of \code{\link{brmsformula}} (under "Additional response information").
+#'
 #' @name addition-terms
-#' 
+#' @aliases se weights trials thres cat dec cens trunc 
+#' @aliases index rate subset vreal vint
+#'
 #' @param x A vector; usually a variable defined in the data. Allowed values
 #'   depend on the function: \code{resp_se} and \code{resp_weights} require
 #'   positive numeric values. \code{resp_trials}, \code{resp_thres}, and
@@ -15,7 +19,8 @@
 #'   \code{FALSE} and \code{TRUE}. \code{resp_cens} requires \code{'left'},
 #'   \code{'none'}, \code{'right'}, and \code{'interval'} (or equivalently
 #'   \code{-1}, \code{0}, \code{1}, and \code{2}) to indicate left, no, right,
-#'   or interval censoring.
+#'   or interval censoring. \code{resp_index} does not make any requirements
+#'   other than the value being unique for each observation.
 #' @param sigma Logical; Indicates whether the residual standard deviation
 #'  parameter \code{sigma} should be included in addition to the known
 #'  measurement error. Defaults to \code{FALSE} for backwards compatibility,
@@ -23,12 +28,12 @@
 #' @param scale Logical; Indicates whether weights should be scaled
 #'  so that the average weight equals one. Defaults to \code{FALSE}.
 #' @param y2 A vector specifying the upper bounds in interval censoring.
-#'  Will be ignored for non-interval censored observations. However, it 
+#'  Will be ignored for non-interval censored observations. However, it
 #'  should NOT be \code{NA} even for non-interval censored observations to
 #'  avoid accidental exclusion of these observations.
-#' @param lb A numeric vector or single numeric value specifying 
+#' @param lb A numeric vector or single numeric value specifying
 #'   the lower truncation bound.
-#' @param ub A numeric vector or single numeric value specifying 
+#' @param ub A numeric vector or single numeric value specifying
 #'   the upper truncation bound.
 #' @param sdy Optional known measurement error of the response
 #'   treated as standard deviation. If specified, handles
@@ -37,24 +42,26 @@
 #' @param denom A vector of positive numeric values specifying
 #'   the denominator values from which the response rates are computed.
 #' @param gr A vector of grouping indicators.
-#' @param ... For \code{resp_vreal}, vectors of real values. 
-#'   For \code{resp_vint}, vectors of integer values.  
+#' @param ... For \code{resp_vreal}, vectors of real values.
+#'   For \code{resp_vint}, vectors of integer values. In Stan,
+#'   these variables will be named \code{vreal1}, \code{vreal2}, ...,
+#'   and \code{vint1}, \code{vint2}, ..., respectively.
 #'
 #' @return A list of additional response information to be processed further
 #'   by \pkg{brms}.
 #'
-#' @details 
+#' @details
 #'   These functions are almost solely useful when
 #'   called in formulas passed to the \pkg{brms} package.
 #'   Within formulas, the \code{resp_} prefix may be omitted.
 #'   More information is given in the 'Details' section
-#'   of \code{\link{brmsformula}}.
-#'   
-#' @seealso 
-#'   \code{\link{brm}}, 
-#'   \code{\link{brmsformula}}   
-#'  
-#' @examples 
+#'   of \code{\link{brmsformula}} (under "Additional response information").
+#'
+#' @seealso
+#'   \code{\link{brm}},
+#'   \code{\link{brmsformula}}
+#'
+#' @examples
 #' \dontrun{
 #' ## Random effects meta-analysis
 #' nstudies <- 20
@@ -65,7 +72,7 @@
 #' fit1 <- brm(outcomes | se(sei, sigma = TRUE) ~ 1,
 #'             data = data1)
 #' summary(fit1)
-#' 
+#'
 #' ## Probit regression using the binomial family
 #' n <- sample(1:10, 100, TRUE)  # number of trials
 #' success <- rbinom(100, size = n, prob = 0.4)
@@ -74,19 +81,19 @@
 #' fit2 <- brm(success | trials(n) ~ x, data = data2,
 #'             family = binomial("probit"))
 #' summary(fit2)
-#' 
-#' ## Survival regression modeling the time between the first 
+#'
+#' ## Survival regression modeling the time between the first
 #' ## and second recurrence of an infection in kidney patients.
-#' fit3 <- brm(time | cens(censored) ~ age * sex + disease + (1|patient), 
+#' fit3 <- brm(time | cens(censored) ~ age * sex + disease + (1|patient),
 #'             data = kidney, family = lognormal())
 #' summary(fit3)
-#' 
-#' ## Poisson model with truncated counts  
-#' fit4 <- brm(count | trunc(ub = 104) ~ zBase * Trt, 
+#'
+#' ## Poisson model with truncated counts
+#' fit4 <- brm(count | trunc(ub = 104) ~ zBase * Trt,
 #'             data = epilepsy, family = poisson())
 #' summary(fit4)
 #' }
-#'   
+#'
 NULL
 
 #' @rdname addition-terms
@@ -134,7 +141,7 @@ resp_cat <- function(x) {
   thres <- deparse(substitute(x))
   str_add(thres) <- " - 1"
   class_resp_special(
-    "thres", call = match.call(), 
+    "thres", call = match.call(),
     vars = nlist(thres, gr = "NA")
   )
 }
@@ -169,6 +176,13 @@ resp_mi <- function(sdy = NA) {
   class_resp_special("mi", call = match.call(), vars = nlist(sdy))
 }
 
+#' @rdname addition-terms
+#' @export
+resp_index <- function(x) {
+  index <- deparse(substitute(x))
+  class_resp_special("index", call = match.call(), vars = nlist(index))
+}
+
 #' @rdname addition-terms
 #' @export
 resp_rate <- function(denom) {
@@ -218,7 +232,7 @@ eval_rhs <- function(formula, data = NULL) {
 }
 
 # get expression for a variable of an addition term
-# @param x list with potentail $adforms elements
+# @param x list with potential $adforms elements
 # @param ad name of the addition term
 # @param target name of the element to extract
 # @type type of the element to extract
@@ -271,7 +285,7 @@ get_ad_vars.mvbrmsterms <- function(x, ad, ...) {
 # @param x vector of censoring indicators
 # @return transformed vector of censoring indicators
 prepare_cens <- function(x) {
-  .prepare_cens <- function(x) {  
+  .prepare_cens <- function(x) {
     stopifnot(length(x) == 1L)
     regx <- paste0("^", x)
     if (grepl(regx, "left")) {
@@ -312,7 +326,7 @@ get_cens <- function(bterms, data, resp = NULL) {
 # @param incl_family include the family in the derivation of the bounds?
 # @param stan return bounds in form of Stan syntax?
 # @return a list with elements 'lb' and 'ub' or corresponding Stan code
-trunc_bounds <- function(bterms, data = NULL, incl_family = FALSE, 
+trunc_bounds <- function(bterms, data = NULL, incl_family = FALSE,
                          stan = FALSE, ...) {
   stopifnot(is.brmsterms(bterms))
   if (is.formula(bterms$adforms$trunc)) {
@@ -355,5 +369,62 @@ has_subset <- function(bterms) {
   } else {
     out <- FALSE
   }
-  out 
+  out
+}
+
+# construct a list of indices for cross-formula referencing
+tidy_index <- function(x, data) {
+  out <- .tidy_index(x, data)
+  if (is.brmsterms(x)) {
+    # ensure consistent format for both uni- and multivariate models
+    out <- list(out)
+    names(out)[1] <- terms_resp(x$respform)
+  }
+  out
+}
+
+# internal version of tidy_index
+.tidy_index <- function(x, ...) {
+  UseMethod(".tidy_index")
+}
+
+#' @export
+.tidy_index.brmsterms <- function(x, data, ...) {
+  out <- get_ad_values(x, "index", "index", data)
+  if (is.null(out)) {
+    return(NULL)
+  }
+  if (has_subset(x)) {
+    subset <- as.logical(get_ad_values(x, "subset", "subset", data))
+    out <- out[subset]
+    attr(out, "subset") <- TRUE
+  }
+  if (anyNA(out)) {
+    stop2("NAs are not allowed in 'index' variables.")
+  }
+  if (anyDuplicated(out)) {
+    stop2("Index of response '", names(out), "' contains duplicated values.")
+  }
+  out
+}
+
+#' @export
+.tidy_index.mvbrmsterms <- function(x, data, ...) {
+  lapply(x$terms, .tidy_index, data = data, ...)
+}
+
+# check if cross-formula referencing is possible in subsetted models
+check_cross_formula_indexing <- function(bterms) {
+  sp_terms <- ulapply(get_effect(bterms, "sp"), all_terms)
+  me_terms <- get_matches_expr(regex_sp("me"), sp_terms)
+  if (length(me_terms)) {
+    stop2("Cannot use me() terms in subsetted formulas.")
+  }
+  mi_terms <- get_matches_expr(regex_sp("mi"), sp_terms)
+  idx_vars <- lapply(mi_terms, function(x) eval2(x)$idx)
+  if (any(idx_vars == "NA")) {
+    stop2("mi() terms in subsetted formulas require ",
+          "the 'idx' argument to be specified.")
+  }
+  invisible(TRUE)
 }
