@@ -1,45 +1,46 @@
 #' Data for \pkg{brms} Models
-#' 
+#'
 #' Generate data for \pkg{brms} models to be passed to \pkg{Stan}
 #'
 #' @inheritParams brm
 #' @param ... Other arguments for internal use.
-#' 
-#' @return A named list of objects containing the required data 
-#'   to fit a \pkg{brms} model with \pkg{Stan}. 
-#' 
+#'
+#' @return A named list of objects containing the required data
+#'   to fit a \pkg{brms} model with \pkg{Stan}.
+#'
 #' @author Paul-Christian Buerkner \email{paul.buerkner@@gmail.com}
-#' 
+#'
 #' @examples
-#' sdata1 <- make_standata(rating ~ treat + period + carry + (1|subject), 
+#' sdata1 <- make_standata(rating ~ treat + period + carry + (1|subject),
 #'                         data = inhaler, family = "cumulative")
 #' str(sdata1)
-#' 
+#'
 #' sdata2 <- make_standata(count ~ zAge + zBase * Trt + (1|patient),
 #'                         data = epilepsy, family = "poisson")
 #' str(sdata2)
-#'          
+#'
 #' @export
-make_standata <- function(formula, data, family = gaussian(), prior = NULL, 
-                          autocor = NULL, data2 = NULL, cov_ranef = NULL, 
+make_standata <- function(formula, data, family = gaussian(), prior = NULL,
+                          autocor = NULL, data2 = NULL, cov_ranef = NULL,
                           sample_prior = "no", stanvars = NULL,
-                          threads = NULL, knots = NULL, ...) {
+                          threads = getOption("brms.threads", NULL),
+                          knots = NULL, ...) {
 
   if (is.brmsfit(formula)) {
     stop2("Use 'standata' to extract Stan data from 'brmsfit' objects.")
   }
   formula <- validate_formula(
-    formula, data = data, family = family, 
+    formula, data = data, family = family,
     autocor = autocor, cov_ranef = cov_ranef
   )
   bterms <- brmsterms(formula)
   data2 <- validate_data2(
-    data2, bterms = bterms, 
+    data2, bterms = bterms,
     get_data2_autocor(formula),
     get_data2_cov_ranef(formula)
   )
   data <- validate_data(
-    data, bterms = bterms, 
+    data, bterms = bterms,
     knots = knots, data2 = data2
   )
   prior <- .validate_prior(
@@ -63,11 +64,11 @@ make_standata <- function(formula, data, family = gaussian(), prior = NULL,
 # @param basis original Stan data as prepared by 'standata_basis'
 # @param ... currently ignored
 # @return names list of data passed to Stan
-.make_standata <- function(bterms, data, prior, stanvars, data2, 
-                           threads = threading(), check_response = TRUE, 
+.make_standata <- function(bterms, data, prior, stanvars, data2,
+                           threads = threading(), check_response = TRUE,
                            only_response = FALSE, internal = FALSE,
                            basis = NULL, ...) {
-  
+
   check_response <- as_one_logical(check_response)
   only_response <- as_one_logical(only_response)
   internal <- as_one_logical(internal)
@@ -80,9 +81,10 @@ make_standata <- function(formula, data, family = gaussian(), prior = NULL,
   if (!only_response) {
     ranef <- tidy_ranef(bterms, data, old_levels = basis$levels)
     meef <- tidy_meef(bterms, data, old_levels = basis$levels)
+    index <- tidy_index(bterms, data)
     c(out) <- data_predictor(
       bterms, data = data, prior = prior, data2 = data2,
-      ranef = ranef, basis = basis
+      ranef = ranef, index = index, basis = basis
     )
     c(out) <- data_gr_global(ranef, data2 = data2)
     c(out) <- data_Xme(meef, data = data)
@@ -99,7 +101,7 @@ make_standata <- function(formula, data, family = gaussian(), prior = NULL,
     stanvars <- subset_stanvars(stanvars, block = "data")
     inv_names <- intersect(names(stanvars), names(out))
     if (length(inv_names)) {
-      stop2("Cannot overwrite existing variables: ", 
+      stop2("Cannot overwrite existing variables: ",
             collapse_comma(inv_names))
     }
     out[names(stanvars)] <- lapply(stanvars, "[[", "sdata")
@@ -108,58 +110,58 @@ make_standata <- function(formula, data, family = gaussian(), prior = NULL,
     # allows to recover the original order of the data
     attr(out, "old_order") <- attr(data, "old_order")
     # ensures current grouping levels are known in post-processing
-    attr(out, "levels") <- get_levels(
-      tidy_meef(bterms, data), tidy_ranef(bterms, data)
-    )
+    ranef_new <- tidy_ranef(bterms, data)
+    meef_new <- tidy_meef(bterms, data)
+    attr(out, "levels") <- get_levels(ranef_new, meef_new)
   }
-  structure(out, class = "standata")
+  structure(out, class = c("standata", "list"))
 }
 
 #' Extract data passed to Stan
-#' 
+#'
 #' Extract all data that was used by Stan to fit the model.
-#' 
+#'
 #' @aliases standata.brmsfit
-#' 
+#'
 #' @param object An object of class \code{brmsfit}.
 #' @param ... More arguments passed to \code{\link{make_standata}}
 #'   and \code{\link{validate_newdata}}.
 #' @inheritParams prepare_predictions
-#' 
+#'
 #' @return A named list containing the data originally passed to Stan.
-#' 
+#'
 #' @export
-standata.brmsfit <- function(object, newdata = NULL, re_formula = NULL, 
+standata.brmsfit <- function(object, newdata = NULL, re_formula = NULL,
                              newdata2 = NULL, new_objects = NULL,
                              incl_autocor = TRUE, ...) {
-  
+
   object <- restructure(object)
   # allows functions to fall back to old default behavior
   # which was used when originally fitting the model
   options(.brmsfit_version = object$version$brms)
   on.exit(options(.brmsfit_version = NULL))
-  
+
   object <- exclude_terms(object, incl_autocor = incl_autocor)
   formula <- update_re_terms(object$formula, re_formula)
   bterms <- brmsterms(formula)
-  
+
   newdata2 <- use_alias(newdata2, new_objects)
   data2 <- current_data2(object, newdata2)
   data <- current_data(
-    object, newdata, newdata2 = data2, 
+    object, newdata, newdata2 = data2,
     re_formula = re_formula, ...
   )
   stanvars <- add_newdata_stanvars(object$stanvars, data2)
-  
+
   basis <- NULL
   if (!is.null(newdata)) {
-    # 'basis' contains information from original Stan data 
+    # 'basis' contains information from original Stan data
     # required to correctly predict from new data
     basis <- standata_basis(bterms, data = object$data)
   }
   .make_standata(
     bterms, data = data, prior = object$prior,
-    data2 = data2, stanvars = stanvars, 
+    data2 = data2, stanvars = stanvars,
     threads = object$threads, basis = basis, ...
   )
 }
@@ -242,7 +244,7 @@ standata_basis_sm <- function(x, data, ...) {
     # the spline penalty has changed in 2.8.7 (#646)
     diagonal.penalty <- !require_old_default("2.8.7")
     gam_args <- list(
-      data = data, knots = knots, 
+      data = data, knots = knots,
       absorb.cons = TRUE, modCon = 3,
       diagonal.penalty = diagonal.penalty
     )
@@ -282,7 +284,7 @@ standata_basis_ac <- function(x, data, ...) {
     gr <- get_ac_vars(x, "gr", class = "car")
     stopifnot(length(gr) <= 1L)
     if (isTRUE(nzchar(gr))) {
-      out$locations <- levels(factor(get(gr, data)))
+      out$locations <- extract_levels(get(gr, data))
     } else {
       out$locations <- NA
     }
