@@ -1,5 +1,5 @@
 #' Draws from the Posterior Predictive Distribution
-#' 
+#'
 #' Compute posterior draws of the posterior predictive distribution. Can be
 #' performed for the data used to fit the model (posterior predictive checks) or
 #' for new data. By definition, these draws have higher variance than draws
@@ -7,36 +7,36 @@
 #' \code{\link{posterior_epred.brmsfit}}. This is because the residual error
 #' is incorporated in \code{posterior_predict}. However, the estimated means of
 #' both methods averaged across draws should be very similar.
-#' 
+#'
 #' @inheritParams prepare_predictions
 #' @param object An object of class \code{brmsfit}.
 #' @param re.form Alias of \code{re_formula}.
-#' @param transform (Deprecated) A function or a character string naming 
+#' @param transform (Deprecated) A function or a character string naming
 #'   a function to be applied on the predicted responses
-#'   before summary statistics are computed. 
-#' @param negative_rt Only relevant for Wiener diffusion models. 
+#'   before summary statistics are computed.
+#' @param negative_rt Only relevant for Wiener diffusion models.
 #'   A flag indicating whether response times of responses
 #'   on the lower boundary should be returned as negative values.
 #'   This allows to distinguish responses on the upper and
 #'   lower boundary. Defaults to \code{FALSE}.
-#' @param sort Logical. Only relevant for time series models. 
-#'  Indicating whether to return predicted values in the original 
-#'  order (\code{FALSE}; default) or in the order of the 
-#'  time series (\code{TRUE}). 
-#' @param ntrys Parameter used in rejection sampling 
-#'   for truncated discrete models only 
+#' @param sort Logical. Only relevant for time series models.
+#'  Indicating whether to return predicted values in the original
+#'  order (\code{FALSE}; default) or in the order of the
+#'  time series (\code{TRUE}).
+#' @param ntrys Parameter used in rejection sampling
+#'   for truncated discrete models only
 #'   (defaults to \code{5}). See Details for more information.
 #' @param cores Number of cores (defaults to \code{1}). On non-Windows systems,
 #'   this argument can be set globally via the \code{mc.cores} option.
 #' @param ... Further arguments passed to \code{\link{prepare_predictions}}
 #'   that control several aspects of data validation and prediction.
-#' 
+#'
 #' @return An \code{array} of predicted response values. In univariate models,
 #'   the output is as an S x N matrix, where S is the number of posterior
 #'   draws and N is the number of observations. In multivariate models, an
 #'   additional dimension is added to the output which indexes along the
 #'   different response variables.
-#'   
+#'
 #' @template details-newdata-na
 #' @template details-allow_new_levels
 #' @details For truncated discrete models only: In the absence of any general
@@ -49,21 +49,21 @@
 #'   invalid, the closest boundary is used, instead. If there are more than a
 #'   few of these pathological cases, a warning will occur suggesting to
 #'   increase argument \code{ntrys}.
-#' 
-#' @examples 
+#'
+#' @examples
 #' \dontrun{
 #' ## fit a model
-#' fit <- brm(time | cens(censored) ~ age + sex + (1 + age || patient), 
-#'            data = kidney, family = "exponential", inits = "0")
-#' 
+#' fit <- brm(time | cens(censored) ~ age + sex + (1 + age || patient),
+#'            data = kidney, family = "exponential", init = "0")
+#'
 #' ## predicted responses
 #' pp <- posterior_predict(fit)
 #' str(pp)
-#' 
+#'
 #' ## predicted responses excluding the group-level effect of age
 #' pp <- posterior_predict(fit, re_formula = ~ (1 | patient))
 #' str(pp)
-#' 
+#'
 #' ## predicted responses of patient 1 for new data
 #' newdata <- data.frame(
 #'   sex = factor(c("male", "female")),
@@ -73,16 +73,16 @@
 #' pp <- posterior_predict(fit, newdata = newdata)
 #' str(pp)
 #' }
-#' 
+#'
 #' @aliases posterior_predict
 #' @method posterior_predict brmsfit
 #' @importFrom rstantools posterior_predict
 #' @export
 #' @export posterior_predict
 posterior_predict.brmsfit <- function(
-  object, newdata = NULL, re_formula = NULL, re.form = NULL, 
-  transform = NULL, resp = NULL, negative_rt = FALSE, 
-  ndraws = NULL, draw_ids = NULL, sort = FALSE, ntrys = 5, 
+  object, newdata = NULL, re_formula = NULL, re.form = NULL,
+  transform = NULL, resp = NULL, negative_rt = FALSE,
+  ndraws = NULL, draw_ids = NULL, sort = FALSE, ntrys = 5,
   cores = NULL, ...
 ) {
   cl <- match.call()
@@ -92,11 +92,11 @@ posterior_predict.brmsfit <- function(
   contains_draws(object)
   object <- restructure(object)
   prep <- prepare_predictions(
-    object, newdata = newdata, re_formula = re_formula, resp = resp, 
+    object, newdata = newdata, re_formula = re_formula, resp = resp,
     ndraws = ndraws, draw_ids = draw_ids, check_response = FALSE, ...
   )
   posterior_predict(
-    prep, transform = transform, sort = sort, ntrys = ntrys, 
+    prep, transform = transform, sort = sort, ntrys = ntrys,
     negative_rt = negative_rt, cores = cores, summary = FALSE
   )
 }
@@ -117,14 +117,14 @@ posterior_predict.mvbrmsprep <- function(object, ...) {
 
 #' @export
 posterior_predict.brmsprep <- function(object, transform = NULL, sort = FALSE,
-                                       summary = FALSE, robust = FALSE, 
-                                       probs = c(0.025, 0.975), 
+                                       summary = FALSE, robust = FALSE,
+                                       probs = c(0.025, 0.975),
                                        cores = NULL, ...) {
   summary <- as_one_logical(summary)
   cores <- validate_cores_post_processing(cores)
   if (is.customfamily(object$family)) {
     # ensure that the method can be found during parallel execution
-    object$family$posterior_predict <- 
+    object$family$posterior_predict <-
       custom_family_method(object$family, "posterior_predict")
   }
   for (nlp in names(object$nlpars)) {
@@ -146,7 +146,7 @@ posterior_predict.brmsprep <- function(object, transform = NULL, sort = FALSE,
     out <- aperm(out, perm = c(1, 3, 2))
     dimnames(out)[[3]] <- object$cats
   } else {
-    out <- do_call(cbind, out) 
+    out <- do_call(cbind, out)
   }
   colnames(out) <- rownames(out) <- NULL
   if (use_int(object$family)) {
@@ -155,10 +155,10 @@ posterior_predict.brmsprep <- function(object, transform = NULL, sort = FALSE,
     )
   }
   out <- reorder_obs(out, object$old_order, sort = sort)
-  # transform predicted response draws before summarizing them 
+  # transform predicted response draws before summarizing them
   if (!is.null(transform)) {
     # deprecated as of brms 2.12.3
-    warning2("Argument 'transform' is deprecated ", 
+    warning2("Argument 'transform' is deprecated ",
              "and will be removed in the future.")
     out <- do_call(transform, list(out))
   }
@@ -179,25 +179,25 @@ posterior_predict.brmsprep <- function(object, transform = NULL, sort = FALSE,
 }
 
 #' Draws from the Posterior Predictive Distribution
-#' 
+#'
 #' This method is an alias of \code{\link{posterior_predict.brmsfit}}
 #' with additional arguments for obtaining summaries of the computed draws.
-#' 
+#'
 #' @inheritParams posterior_predict.brmsfit
 #' @param summary Should summary statistics be returned
 #'  instead of the raw values? Default is \code{TRUE}.
-#' @param robust If \code{FALSE} (the default) the mean is used as 
-#'  the measure of central tendency and the standard deviation as 
-#'  the measure of variability. If \code{TRUE}, the median and the 
+#' @param robust If \code{FALSE} (the default) the mean is used as
+#'  the measure of central tendency and the standard deviation as
+#'  the measure of variability. If \code{TRUE}, the median and the
 #'  median absolute deviation (MAD) are applied instead.
 #'  Only used if \code{summary} is \code{TRUE}.
-#' @param probs  The percentiles to be computed by the \code{quantile} 
-#'  function. Only used if \code{summary} is \code{TRUE}. 
-#'  
+#' @param probs  The percentiles to be computed by the \code{quantile}
+#'  function. Only used if \code{summary} is \code{TRUE}.
+#'
 #' @return An \code{array} of predicted response values.
-#'   If \code{summary = FALSE} the output resembles those of 
+#'   If \code{summary = FALSE} the output resembles those of
 #'   \code{\link{posterior_predict.brmsfit}}.
-#' 
+#'
 #'   If \code{summary = TRUE} the output depends on the family: For categorical
 #'   and ordinal families, the output is an N x C matrix, where N is the number
 #'   of observations, C is the number of categories, and the values are
@@ -208,24 +208,24 @@ posterior_predict.brmsprep <- function(object, transform = NULL, sort = FALSE,
 #'   \code{Est.Error} column contains uncertainty estimates (either standard
 #'   deviation or median absolute deviation depending on argument
 #'   \code{robust}). The remaining columns starting with \code{Q} contain
-#'   quantile estimates as specified via argument \code{probs}.  
-#'   
-#' @seealso \code{\link{posterior_predict.brmsfit}} 
-#'  
-#' @examples 
+#'   quantile estimates as specified via argument \code{probs}.
+#'
+#' @seealso \code{\link{posterior_predict.brmsfit}}
+#'
+#' @examples
 #' \dontrun{
 #' ## fit a model
-#' fit <- brm(time | cens(censored) ~ age + sex + (1 + age || patient), 
-#'            data = kidney, family = "exponential", inits = "0")
-#' 
+#' fit <- brm(time | cens(censored) ~ age + sex + (1 + age || patient),
+#'            data = kidney, family = "exponential", init = "0")
+#'
 #' ## predicted responses
 #' pp <- predict(fit)
 #' head(pp)
-#' 
+#'
 #' ## predicted responses excluding the group-level effect of age
 #' pp <- predict(fit, re_formula = ~ (1 | patient))
 #' head(pp)
-#' 
+#'
 #' ## predicted responses of patient 1 for new data
 #' newdata <- data.frame(
 #'   sex = factor(c("male", "female")),
@@ -234,22 +234,22 @@ posterior_predict.brmsprep <- function(object, transform = NULL, sort = FALSE,
 #' )
 #' predict(fit, newdata = newdata)
 #' }
-#'  
+#'
 #' @export
 predict.brmsfit <- function(object, newdata = NULL, re_formula = NULL,
-                            transform = NULL, resp = NULL, 
-                            negative_rt = FALSE, ndraws = NULL, draw_ids = NULL, 
-                            sort = FALSE, ntrys = 5, cores = NULL, summary = TRUE, 
+                            transform = NULL, resp = NULL,
+                            negative_rt = FALSE, ndraws = NULL, draw_ids = NULL,
+                            sort = FALSE, ntrys = 5, cores = NULL, summary = TRUE,
                             robust = FALSE, probs = c(0.025, 0.975), ...) {
   contains_draws(object)
   object <- restructure(object)
   prep <- prepare_predictions(
-    object, newdata = newdata, re_formula = re_formula, resp = resp, 
+    object, newdata = newdata, re_formula = re_formula, resp = resp,
     ndraws = ndraws, draw_ids = draw_ids, check_response = FALSE, ...
   )
   posterior_predict(
-    prep, transform = transform, ntrys = ntrys, negative_rt = negative_rt, 
-    sort = sort, cores = cores, summary = summary, robust = robust, 
+    prep, transform = transform, ntrys = ntrys, negative_rt = negative_rt,
+    sort = sort, cores = cores, summary = summary, robust = robust,
     probs = probs
   )
 }
@@ -257,23 +257,23 @@ predict.brmsfit <- function(object, newdata = NULL, re_formula = NULL,
 #' Predictive Intervals
 #'
 #' Compute intervals from the posterior predictive distribution.
-#' 
+#'
 #' @aliases predictive_interval
-#' 
+#'
 #' @param object An \R object of class \code{brmsfit}.
 #' @param prob A number p (0 < p < 1) indicating the desired probability mass to
 #'   include in the intervals. Defaults to \code{0.9}.
 #' @param ... Further arguments passed to \code{\link{posterior_predict}}.
-#' 
+#'
 #' @return A matrix with 2 columns for the lower and upper bounds of the
 #'   intervals, respectively, and as many rows as observations being predicted.
-#' 
-#' @examples 
+#'
+#' @examples
 #' \dontrun{
 #' fit <- brm(count ~ zBase, data = epilepsy, family = poisson())
 #' predictive_interval(fit)
 #' }
-#' 
+#'
 #' @importFrom rstantools predictive_interval
 #' @export predictive_interval
 #' @export
@@ -304,7 +304,7 @@ validate_pp_method <- function(method) {
 # ------------------- family specific posterior_predict methods ---------------------
 # All posterior_predict_<family> functions have the same arguments structure
 # @param i index of the observatio for which to compute pp values
-# @param prep A named list returned by prepare_predictions containing 
+# @param prep A named list returned by prepare_predictions containing
 #   all required data and posterior draws
 # @param ... ignored arguments
 # @param A vector of length prep$ndraws containing draws
@@ -327,7 +327,7 @@ posterior_predict_student <- function(i, prep, ntrys = 5, ...) {
   sigma <- get_dpar(prep, "sigma", i = i)
   sigma <- add_sigma_se(sigma, prep, i = i)
   rcontinuous(
-    n = prep$ndraws, dist = "student_t", 
+    n = prep$ndraws, dist = "student_t",
     df = nu, mu = mu, sigma = sigma,
     lb = prep$data$lb[i], ub = prep$data$ub[i],
     ntrys = ntrys
@@ -337,7 +337,7 @@ posterior_predict_student <- function(i, prep, ntrys = 5, ...) {
 posterior_predict_lognormal <- function(i, prep, ntrys = 5, ...) {
   rcontinuous(
     n = prep$ndraws, dist = "lnorm",
-    meanlog = get_dpar(prep, "mu", i = i), 
+    meanlog = get_dpar(prep, "mu", i = i),
     sdlog = get_dpar(prep, "sigma", i = i),
     lb = prep$data$lb[i], ub = prep$data$ub[i],
     ntrys = ntrys
@@ -347,7 +347,7 @@ posterior_predict_lognormal <- function(i, prep, ntrys = 5, ...) {
 posterior_predict_shifted_lognormal <- function(i, prep, ntrys = 5, ...) {
   rcontinuous(
     n = prep$ndraws, dist = "shifted_lnorm",
-    meanlog = get_dpar(prep, "mu", i = i), 
+    meanlog = get_dpar(prep, "mu", i = i),
     sdlog = get_dpar(prep, "sigma", i = i),
     shift = get_dpar(prep, "ndt", i = i),
     lb = prep$data$lb[i], ub = prep$data$ub[i],
@@ -483,10 +483,21 @@ posterior_predict_student_fcor <- function(i, prep, ...) {
 
 posterior_predict_binomial <- function(i, prep, ntrys = 5, ...) {
   rdiscrete(
-    n = prep$ndraws, dist = "binom", 
-    size = prep$data$trials[i], 
+    n = prep$ndraws, dist = "binom",
+    size = prep$data$trials[i],
     prob = get_dpar(prep, "mu", i = i),
-    lb = prep$data$lb[i], ub = prep$data$ub[i], 
+    lb = prep$data$lb[i], ub = prep$data$ub[i],
+    ntrys = ntrys
+  )
+}
+
+posterior_predict_beta_binomial <- function(i, prep, ntrys = 5, ...) {
+  rdiscrete(
+    n = prep$ndraws, dist = "beta_binomial",
+    size = prep$data$trials[i],
+    mu = get_dpar(prep, "mu", i = i),
+    phi = get_dpar(prep, "phi", i = i),
+    lb = prep$data$lb[i], ub = prep$data$ub[i],
     ntrys = ntrys
   )
 }
@@ -540,7 +551,7 @@ posterior_predict_geometric <- function(i, prep, ntrys = 5, ...) {
   rdiscrete(
     n = prep$ndraws, dist = "nbinom",
     mu = mu, size = shape,
-    lb = prep$data$lb[i], ub = prep$data$ub[i], 
+    lb = prep$data$lb[i], ub = prep$data$ub[i],
     ntrys = ntrys
   )
 }
@@ -548,7 +559,7 @@ posterior_predict_geometric <- function(i, prep, ntrys = 5, ...) {
 posterior_predict_discrete_weibull <- function(i, prep, ntrys = 5, ...) {
   rdiscrete(
     n = prep$ndraws, dist = "discrete_weibull",
-    mu = get_dpar(prep, "mu", i = i), 
+    mu = get_dpar(prep, "mu", i = i),
     shape = get_dpar(prep, "shape", i = i),
     lb = prep$data$lb[i], ub = prep$data$ub[i],
     ntrys = ntrys
@@ -558,7 +569,7 @@ posterior_predict_discrete_weibull <- function(i, prep, ntrys = 5, ...) {
 posterior_predict_com_poisson <- function(i, prep, ntrys = 5, ...) {
   rdiscrete(
     n = prep$ndraws, dist = "com_poisson",
-    mu = get_dpar(prep, "mu", i = i), 
+    mu = get_dpar(prep, "mu", i = i),
     shape = get_dpar(prep, "shape", i = i),
     lb = prep$data$lb[i], ub = prep$data$ub[i],
     ntrys = ntrys
@@ -587,7 +598,7 @@ posterior_predict_gamma <- function(i, prep, ntrys = 5, ...) {
 
 posterior_predict_weibull <- function(i, prep, ntrys = 5, ...) {
   shape <- get_dpar(prep, "shape", i = i)
-  scale <- get_dpar(prep, "mu", i = i) / gamma(1 + 1 / shape) 
+  scale <- get_dpar(prep, "mu", i = i) / gamma(1 + 1 / shape)
   rcontinuous(
     n = prep$ndraws, dist = "weibull",
     shape = shape, scale = scale,
@@ -609,7 +620,7 @@ posterior_predict_frechet <- function(i, prep, ntrys = 5, ...) {
 
 posterior_predict_gen_extreme_value <- function(i, prep, ntrys = 5, ...) {
   rcontinuous(
-    n = prep$ndraws, dist = "gen_extreme_value", 
+    n = prep$ndraws, dist = "gen_extreme_value",
     sigma = get_dpar(prep, "sigma", i = i),
     xi = get_dpar(prep, "xi", i = i),
     mu = get_dpar(prep, "mu", i = i),
@@ -621,7 +632,7 @@ posterior_predict_gen_extreme_value <- function(i, prep, ntrys = 5, ...) {
 posterior_predict_inverse.gaussian <- function(i, prep, ntrys = 5, ...) {
   rcontinuous(
     n = prep$ndraws, dist = "inv_gaussian",
-    mu = get_dpar(prep, "mu", i = i), 
+    mu = get_dpar(prep, "mu", i = i),
     shape = get_dpar(prep, "shape", i = i),
     lb = prep$data$lb[i], ub = prep$data$ub[i],
     ntrys = ntrys
@@ -631,7 +642,7 @@ posterior_predict_inverse.gaussian <- function(i, prep, ntrys = 5, ...) {
 posterior_predict_exgaussian <- function(i, prep, ntrys = 5, ...) {
   rcontinuous(
     n = prep$ndraws, dist = "exgaussian",
-    mu = get_dpar(prep, "mu", i = i), 
+    mu = get_dpar(prep, "mu", i = i),
     sigma = get_dpar(prep, "sigma", i = i),
     beta = get_dpar(prep, "beta", i = i),
     lb = prep$data$lb[i], ub = prep$data$ub[i],
@@ -642,8 +653,8 @@ posterior_predict_exgaussian <- function(i, prep, ntrys = 5, ...) {
 posterior_predict_wiener <- function(i, prep, negative_rt = FALSE, ntrys = 5,
                                      ...) {
   out <- rcontinuous(
-    n = 1, dist = "wiener", 
-    delta = get_dpar(prep, "mu", i = i), 
+    n = 1, dist = "wiener",
+    delta = get_dpar(prep, "mu", i = i),
     alpha = get_dpar(prep, "bs", i = i),
     tau = get_dpar(prep, "ndt", i = i),
     beta = get_dpar(prep, "bias", i = i),
@@ -662,7 +673,7 @@ posterior_predict_beta <- function(i, prep, ntrys = 5, ...) {
   mu <- get_dpar(prep, "mu", i = i)
   phi <- get_dpar(prep, "phi", i = i)
   rcontinuous(
-    n = prep$ndraws, dist = "beta", 
+    n = prep$ndraws, dist = "beta",
     shape1 = mu * phi, shape2 = (1 - mu) * phi,
     lb = prep$data$lb[i], ub = prep$data$ub[i],
     ntrys = ntrys
@@ -672,7 +683,7 @@ posterior_predict_beta <- function(i, prep, ntrys = 5, ...) {
 posterior_predict_von_mises <- function(i, prep, ntrys = 5, ...) {
   rcontinuous(
     n = prep$ndraws, dist = "von_mises",
-    mu = get_dpar(prep, "mu", i = i), 
+    mu = get_dpar(prep, "mu", i = i),
     kappa = get_dpar(prep, "kappa", i = i),
     lb = prep$data$lb[i], ub = prep$data$ub[i],
     ntrys = ntrys
@@ -682,7 +693,7 @@ posterior_predict_von_mises <- function(i, prep, ntrys = 5, ...) {
 posterior_predict_asym_laplace <- function(i, prep, ntrys = 5, ...) {
   rcontinuous(
     n = prep$ndraws, dist = "asym_laplace",
-    mu = get_dpar(prep, "mu", i = i), 
+    mu = get_dpar(prep, "mu", i = i),
     sigma = get_dpar(prep, "sigma", i = i),
     quantile = get_dpar(prep, "quantile", i = i),
     lb = prep$data$lb[i], ub = prep$data$ub[i],
@@ -695,10 +706,10 @@ posterior_predict_zero_inflated_asym_laplace <- function(i, prep, ntrys = 5,
   zi <- get_dpar(prep, "zi", i = i)
   tmp <- runif(prep$ndraws, 0, 1)
   ifelse(
-    tmp < zi, 0, 
+    tmp < zi, 0,
     rcontinuous(
       n = prep$ndraws, dist = "asym_laplace",
-      mu = get_dpar(prep, "mu", i = i), 
+      mu = get_dpar(prep, "mu", i = i),
       sigma = get_dpar(prep, "sigma", i = i),
       quantile = get_dpar(prep, "quantile", i = i),
       lb = prep$data$lb[i], ub = prep$data$ub[i],
@@ -714,7 +725,7 @@ posterior_predict_cox <- function(i, prep, ...) {
 
 posterior_predict_hurdle_poisson <- function(i, prep, ...) {
   # theta is the bernoulli hurdle parameter
-  theta <- get_dpar(prep, "hu", i = i) 
+  theta <- get_dpar(prep, "hu", i = i)
   lambda <- get_dpar(prep, "mu", i = i)
   ndraws <- prep$ndraws
   # compare with theta to incorporate the hurdle process
@@ -769,7 +780,7 @@ posterior_predict_zero_inflated_beta <- function(i, prep, ...) {
   # compare with theta to incorporate the hurdle process
   hu <- runif(prep$ndraws, 0, 1)
   ifelse(
-    hu < theta, 0, 
+    hu < theta, 0,
     rbeta(prep$ndraws, shape1 = mu * phi, shape2 = (1 - mu) * phi)
   )
 }
@@ -781,7 +792,7 @@ posterior_predict_zero_one_inflated_beta <- function(i, prep, ...) {
   phi <- get_dpar(prep, "phi", i = i)
   hu <- runif(prep$ndraws, 0, 1)
   one_or_zero <- runif(prep$ndraws, 0, 1)
-  ifelse(hu < zoi, 
+  ifelse(hu < zoi,
     ifelse(one_or_zero < coi, 1, 0),
     rbeta(prep$ndraws, shape1 = mu * phi, shape2 = (1 - mu) * phi)
   )
@@ -809,7 +820,7 @@ posterior_predict_zero_inflated_negbinomial <- function(i, prep, ...) {
 }
 
 posterior_predict_zero_inflated_binomial <- function(i, prep, ...) {
-  # theta is the bernoulii zero-inflation parameter
+  # theta is the bernoulli zero-inflation parameter
   theta <- get_dpar(prep, "zi", i = i)
   trials <- prep$data$trials[i]
   prob <- get_dpar(prep, "mu", i = i)
@@ -819,26 +830,38 @@ posterior_predict_zero_inflated_binomial <- function(i, prep, ...) {
   ifelse(zi < theta, 0, rbinom(ndraws, size = trials, prob = prob))
 }
 
+posterior_predict_zero_inflated_beta_binomial <- function(i, prep, ...) {
+  # theta is the bernoulli zero-inflation parameter
+  theta <- get_dpar(prep, "zi", i = i)
+  trials <- prep$data$trials[i]
+  mu <- get_dpar(prep, "mu", i = i)
+  phi <- get_dpar(prep, "phi", i = i)
+  ndraws <- prep$ndraws
+  draws <- rbeta_binomial(ndraws, size = trials, mu = mu, phi = phi)
+  # compare with theta to incorporate the zero-inflation process
+  zi <- runif(ndraws, 0, 1)
+  draws[zi < theta] <- 0
+  draws
+}
+
 posterior_predict_categorical <- function(i, prep, ...) {
-  eta <- cblapply(names(prep$dpars), get_dpar, prep = prep, i = i)
-  eta <- insert_refcat(eta, family = prep$family)
+  eta <- get_Mu(prep, i = i)
+  eta <- insert_refcat(eta, refcat = prep$refcat)
   p <- pcategorical(seq_len(prep$data$ncat), eta = eta)
   first_greater(p, target = runif(prep$ndraws, min = 0, max = 1))
 }
 
 posterior_predict_multinomial <- function(i, prep, ...) {
-  eta <- cblapply(names(prep$dpars), get_dpar, prep = prep, i = i)
-  eta <- insert_refcat(eta, family = prep$family)
+  eta <- get_Mu(prep, i = i)
+  eta <- insert_refcat(eta, refcat = prep$refcat)
   p <- dcategorical(seq_len(prep$data$ncat), eta = eta)
   size <- prep$data$trials[i]
-  out <- lapply(seq_rows(p), function(s) t(rmultinom(1, size, p[s, ])))
-  do_call(rbind, out)
+  rblapply(seq_rows(p), function(s) t(rmultinom(1, size, p[s, ])))
 }
 
 posterior_predict_dirichlet <- function(i, prep, ...) {
-  mu_dpars <- str_subset(names(prep$dpars), "^mu")
-  eta <- cblapply(mu_dpars, get_dpar, prep = prep, i = i)
-  eta <- insert_refcat(eta, family = prep$family)
+  eta <- get_Mu(prep, i = i)
+  eta <- insert_refcat(eta, refcat = prep$refcat)
   phi <- get_dpar(prep, "phi", i = i)
   cats <- seq_len(prep$data$ncat)
   alpha <- dcategorical(cats, eta = eta) * phi
@@ -846,11 +869,20 @@ posterior_predict_dirichlet <- function(i, prep, ...) {
 }
 
 posterior_predict_dirichlet2 <- function(i, prep, ...) {
-  mu_dpars <- str_subset(names(prep$dpars), "^mu")
-  mu <- cblapply(mu_dpars, get_dpar, prep = prep, i = i)
+  mu <- get_Mu(prep, i = i)
   rdirichlet(prep$ndraws, alpha = mu)
 }
 
+posterior_predict_logistic_normal <- function(i, prep, ...) {
+  mu <- get_Mu(prep, i = i)
+  Sigma <- get_Sigma(prep, i = i, cor_name = "lncor")
+  .predict <- function(s) {
+    rlogistic_normal(1, mu = mu[s, ], Sigma = Sigma[s, , ],
+                     refcat = prep$refcat)
+  }
+  rblapply(seq_len(prep$ndraws), .predict)
+}
+
 posterior_predict_cumulative <- function(i, prep, ...) {
   posterior_predict_ordinal(i = i, prep = prep)
 }
@@ -865,17 +897,17 @@ posterior_predict_cratio <- function(i, prep, ...) {
 
 posterior_predict_acat <- function(i, prep, ...) {
   posterior_predict_ordinal(i = i, prep = prep)
-}  
+}
 
 posterior_predict_ordinal <- function(i, prep, ...) {
   thres <- subset_thres(prep, i)
   nthres <- NCOL(thres)
   p <- pordinal(
-    seq_len(nthres + 1), 
-    eta = get_dpar(prep, "mu", i = i), 
+    seq_len(nthres + 1),
+    eta = get_dpar(prep, "mu", i = i),
     disc = get_dpar(prep, "disc", i = i),
     thres = thres,
-    family = prep$family$family, 
+    family = prep$family$family,
     link = prep$family$link
   )
   first_greater(p, target = runif(prep$ndraws, min = 0, max = 1))
@@ -959,7 +991,7 @@ rdiscrete <- function(n, dist, ..., lb = NULL, ub = NULL, ntrys = 5) {
     if (is.null(ub)) ub <- Inf
     out <- vector("list", ntrys)
     for (i in seq_along(out)) {
-      # loop of the trys to prevent a mismatch between 'n' 
+      # loop of the trys to prevent a mismatch between 'n'
       # and length of the parameter vectors passed as arguments
       out[[i]] <- as.vector(do_call(rdist, c(list(n), args)))
     }
@@ -978,7 +1010,7 @@ sample_mixture_ids <- function(theta) {
   )
 }
 
-# extract the first valid predicted value per Stan sample per observation 
+# extract the first valid predicted value per Stan sample per observation
 # @param x draws to be check against truncation boundaries
 # @param lb vector of lower bounds
 # @param ub vector of upper bound
@@ -1014,7 +1046,7 @@ check_discrete_trunc_bounds <- function(x, lb = NULL, ub = NULL, thres = 0.01) {
   pct_invalid <- mean(y < lb | y > ub, na.rm = TRUE)
   if (pct_invalid >= thres) {
     warning2(
-      round(pct_invalid * 100), "% of all predicted values ", 
+      round(pct_invalid * 100), "% of all predicted values ",
       "were invalid. Increasing argument 'ntrys' may help."
     )
   }
