@@ -1,25 +1,25 @@
 #' Create a summary of a fitted model represented by a \code{brmsfit} object
 #'
 #' @param object An object of class \code{brmsfit}.
-#' @param priors Logical; Indicating if priors should be included 
+#' @param priors Logical; Indicating if priors should be included
 #'   in the summary. Default is \code{FALSE}.
-#' @param prob A value between 0 and 1 indicating the desired probability 
+#' @param prob A value between 0 and 1 indicating the desired probability
 #'   to be covered by the uncertainty intervals. The default is 0.95.
 #' @param mc_se Logical; Indicating if the uncertainty in \code{Estimate}
 #'   caused by the MCMC sampling should be shown in the summary. Defaults to
 #'   \code{FALSE}.
 #' @param ... Other potential arguments
 #' @inheritParams posterior_summary
-#' 
-#' @details The convergence diagnostics \code{Rhat}, \code{Bulk_ESS}, and 
+#'
+#' @details The convergence diagnostics \code{Rhat}, \code{Bulk_ESS}, and
 #' \code{Tail_ESS} are described in detail in Vehtari et al. (2020).
-#' 
-#' @references 
+#'
+#' @references
 #' Aki Vehtari, Andrew Gelman, Daniel Simpson, Bob Carpenter, and
 #' Paul-Christian Bürkner (2020). Rank-normalization, folding, and
 #' localization: An improved R-hat for assessing convergence of
 #' MCMC. *Bayesian Analysis*. 1–28. dpi:10.1214/20-BA1221
-#' 
+#'
 #' @method summary brmsfit
 #' @importMethodsFrom rstan summary
 #' @importFrom posterior subset_draws summarize_draws
@@ -34,10 +34,10 @@ summary.brmsfit <- function(object, priors = FALSE, prob = 0.95,
   bterms <- brmsterms(object$formula)
   out <- list(
     formula = object$formula,
-    data_name = get_data_name(object$data), 
-    group = unique(object$ranef$group), 
-    nobs = nobs(object), 
-    ngrps = ngrps(object), 
+    data_name = get_data_name(object$data),
+    group = unique(object$ranef$group),
+    nobs = nobs(object),
+    ngrps = ngrps(object),
     autocor = object$autocor,
     prior = empty_prior(),
     algorithm = algorithm(object)
@@ -48,15 +48,16 @@ summary.brmsfit <- function(object, priors = FALSE, prob = 0.95,
     return(out)
   }
   out$chains <- nchains(object)
-  out$iter <- niterations(object) + nwarmup(object)
-  out$warmup <- nwarmup(object)
+  # iterations before thinning
+  out$iter <- object$fit@sim$iter
+  out$warmup <- object$fit@sim$warmup
   out$thin <- nthin(object)
   stan_args <- object$fit@stan_args[[1]]
   out$sampler <- paste0(stan_args$method, "(", stan_args$algorithm, ")")
   if (priors) {
     out$prior <- prior_summary(object, all = FALSE)
   }
-  
+
   # compute a summary for given set of parameters
   # TODO: align names with summary outputs of other methods and packages
   .summary <- function(draws, variables, probs, robust) {
@@ -83,8 +84,8 @@ summary.brmsfit <- function(object, priors = FALSE, prob = 0.95,
       measures$Est.Error <- sd
     }
     c(measures) <- list(
-      quantiles = .quantile, 
-      Rhat = posterior::rhat, 
+      quantiles = .quantile,
+      Rhat = posterior::rhat,
       Bulk_ESS = posterior::ess_bulk,
       Tail_ESS = posterior::ess_tail
     )
@@ -94,18 +95,24 @@ summary.brmsfit <- function(object, priors = FALSE, prob = 0.95,
     out$variable <- NULL
     return(out)
   }
-  
+
   variables <- variables(object)
-  excl_regex <- "^(r|s|z|zs|zgp|Xme|L|Lrescor|prior|lp)(_|$)"
-  variables <- variables[!grepl(excl_regex, variables)]
-  draws <- as_draws_array(object)
+  incl_classes <- c(
+    "b", "bs", "bcs", "bsp", "bmo", "bme", "bmi", "bm", 
+    valid_dpars(object), "delta", "lncor", "rescor", "ar", "ma", 
+    "sderr", "cosy", "lagsar", "errorsar", "car", "sdcar", "rhocar", 
+    "sd", "cor", "df", "sds", "sdgp", "lscale", "simo"
+  )
+  incl_regex <- paste0("^", regex_or(incl_classes), "(_|$)")
+  variables <- variables[grepl(incl_regex, variables)]
+  draws <- as_draws_array(object, variable = variables)
   full_summary <- .summary(draws, variables, probs, robust)
   if (algorithm(object) == "sampling") {
     Rhats <- full_summary[, "Rhat"]
     if (any(Rhats > 1.05, na.rm = TRUE)) {
       warning2(
         "Parts of the model have not converged (some Rhats are > 1.05). ",
-        "Be careful when analysing the results! We recommend running ", 
+        "Be careful when analysing the results! We recommend running ",
         "more iterations and/or setting stronger priors."
       )
     }
@@ -113,25 +120,33 @@ summary.brmsfit <- function(object, priors = FALSE, prob = 0.95,
     adapt_delta <- control_params(object)$adapt_delta
     if (div_trans > 0) {
       warning2(
-        "There were ", div_trans, " divergent transitions after warmup. ", 
+        "There were ", div_trans, " divergent transitions after warmup. ",
         "Increasing adapt_delta above ", adapt_delta, " may help. See ",
         "http://mc-stan.org/misc/warnings.html#divergent-transitions-after-warmup"
       )
     }
   }
-  
+
   # summary of population-level effects
   fe_pars <- variables[grepl(fixef_pars(), variables)]
   out$fixed <- full_summary[fe_pars, , drop = FALSE]
   rownames(out$fixed) <- gsub(fixef_pars(), "", fe_pars)
-  
+
   # summary of family specific parameters
   spec_pars <- c(valid_dpars(object), "delta")
   spec_pars <- paste0(spec_pars, collapse = "|")
   spec_pars <- paste0("^(", spec_pars, ")($|_)")
   spec_pars <- variables[grepl(spec_pars, variables)]
   out$spec_pars <- full_summary[spec_pars, , drop = FALSE]
-  
+  # correlation parameters require renaming to look good in the summary
+  lncor_pars <- variables[grepl("^lncor_", variables)]
+  if (length(lncor_pars)) {
+    lncor_summary <- full_summary[lncor_pars, , drop = FALSE]
+    lncor_pars <- sub("__", ",", sub("__", "(", lncor_pars))
+    rownames(lncor_summary) <- paste0(lncor_pars, ")")
+    out$spec_pars <- rbind(out$spec_pars, lncor_summary)
+  }
+
   # summary of residual correlations
   rescor_pars <- variables[grepl("^rescor_", variables)]
   if (length(rescor_pars)) {
@@ -139,12 +154,12 @@ summary.brmsfit <- function(object, priors = FALSE, prob = 0.95,
     rescor_pars <- sub("__", ",", sub("__", "(", rescor_pars))
     rownames(out$rescor_pars) <- paste0(rescor_pars, ")")
   }
-  
+
   # summary of autocorrelation effects
   cor_pars <- variables[grepl(regex_autocor_pars(), variables)]
   out$cor_pars <- full_summary[cor_pars, , drop = FALSE]
   rownames(out$cor_pars) <- cor_pars
-  
+
   # summary of group-level effects
   for (g in out$group) {
     gregex <- escape_dot(g)
@@ -189,17 +204,17 @@ summary.brmsfit <- function(object, priors = FALSE, prob = 0.95,
 }
 
 #' Print a summary for a fitted model represented by a \code{brmsfit} object
-#' 
+#'
 #' @aliases print.brmssummary
-#' 
+#'
 #' @param x An object of class \code{brmsfit}
-#' @param digits The number of significant digits for printing out the summary; 
+#' @param digits The number of significant digits for printing out the summary;
 #'  defaults to 2. The effective sample size is always rounded to integers.
-#' @param ... Additional arguments that would be passed 
+#' @param ... Additional arguments that would be passed
 #'  to method \code{summary} of \code{brmsfit}.
-#' 
+#'
 #' @seealso \code{\link{summary.brmsfit}}
-#' 
+#'
 #' @export
 print.brmsfit <- function(x, digits = 2, ...) {
   print(summary(x, ...), digits = digits, ...)
@@ -214,7 +229,7 @@ print.brmssummary <- function(x, digits = 2, ...) {
   cat("Formula: ")
   print(x$formula, wsp = 9)
   cat(paste0(
-    "   Data: ", x$data_name, 
+    "   Data: ", x$data_name,
     " (Number of observations: ", x$nobs, ") \n"
   ))
   if (!isTRUE(nzchar(x$sampler))) {
@@ -222,7 +237,7 @@ print.brmssummary <- function(x, digits = 2, ...) {
   } else {
     total_ndraws <- ceiling((x$iter - x$warmup) / x$thin * x$chains)
     cat(paste0(
-      "  Draws: ", x$chains, " chains, each with iter = ", x$iter, 
+      "  Draws: ", x$chains, " chains, each with iter = ", x$iter,
       "; warmup = ", x$warmup, "; thin = ", x$thin, ";\n",
       "         total post-warmup draws = ", total_ndraws, "\n\n"
     ))
@@ -281,7 +296,7 @@ print.brmssummary <- function(x, digits = 2, ...) {
       cat(paste0(
         "For each parameter, Bulk_ESS\n",
         "and Tail_ESS are effective sample size measures, ",
-        "and Rhat is the potential\n", 
+        "and Rhat is the potential\n",
         "scale reduction factor on split chains ",
         "(at convergence, Rhat = 1)."
       ))
@@ -330,38 +345,38 @@ algorithm <- function(x) {
 }
 
 #' Summarize Posterior draws
-#' 
+#'
 #' Summarizes posterior draws based on point estimates (mean or median),
 #' estimation errors (SD or MAD) and quantiles. This function mainly exists to
 #' retain backwards compatibility. It will eventually be replaced by functions
 #' of the \pkg{posterior} package (see examples below).
-#' 
+#'
 #' @param x An \R object.
 #' @inheritParams as.matrix.brmsfit
-#' @param probs The percentiles to be computed by the 
+#' @param probs The percentiles to be computed by the
 #'   \code{\link[stats:quantile]{quantile}} function.
-#' @param robust If \code{FALSE} (the default) the mean is used as 
-#'  the measure of central tendency and the standard deviation as 
-#'  the measure of variability. If \code{TRUE}, the median and the 
+#' @param robust If \code{FALSE} (the default) the mean is used as
+#'  the measure of central tendency and the standard deviation as
+#'  the measure of variability. If \code{TRUE}, the median and the
 #'  median absolute deviation (MAD) are applied instead.
 #' @param ... More arguments passed to or from other methods.
-#' 
+#'
 #' @return A matrix where rows indicate variables
 #' and columns indicate the summary estimates.
-#' 
+#'
 #' @seealso \code{\link[posterior:summarize_draws]{summarize_draws}}
-#'  
-#' @examples 
+#'
+#' @examples
 #' \dontrun{
 #' fit <- brm(time ~ age * sex, data = kidney)
 #' posterior_summary(fit)
-#' 
+#'
 #' # recommended workflow using posterior
 #' library(posterior)
 #' draws <- as_draws_array(fit)
 #' summarise_draws(draws, default_summary_measures())
 #' }
-#' 
+#'
 #' @export
 posterior_summary <- function(x, ...) {
   UseMethod("posterior_summary")
@@ -369,7 +384,7 @@ posterior_summary <- function(x, ...) {
 
 #' @rdname posterior_summary
 #' @export
-posterior_summary.default <- function(x, probs = c(0.025, 0.975), 
+posterior_summary.default <- function(x, probs = c(0.025, 0.975),
                                       robust = FALSE, ...) {
   # TODO: replace with summary functions from posterior
   # TODO: find a way to represent 3D summaries as well
@@ -383,7 +398,7 @@ posterior_summary.default <- function(x, probs = c(0.025, 0.975),
   }
   .posterior_summary <- function(x) {
     do_call(cbind, lapply(
-      coefs, get_estimate, draws = x, 
+      coefs, get_estimate, draws = x,
       probs = probs, na.rm = TRUE
     ))
   }
@@ -391,7 +406,7 @@ posterior_summary.default <- function(x, probs = c(0.025, 0.975),
     # data.frames cause trouble in as.array
     x <- as.matrix(x)
   } else {
-    x <- as.array(x) 
+    x <- as.array(x)
   }
   if (length(dim(x)) == 2L) {
     out <- .posterior_summary(x)
@@ -406,19 +421,19 @@ posterior_summary.default <- function(x, probs = c(0.025, 0.975),
   }
   # TODO: align names with summary outputs of other methods and packages
   colnames(out) <- c("Estimate", "Est.Error", paste0("Q", probs * 100))
-  out  
+  out
 }
 
 #' @rdname posterior_summary
 #' @export
-posterior_summary.brmsfit <- function(x, pars = NA, variable = NULL, 
-                                      probs = c(0.025, 0.975), 
+posterior_summary.brmsfit <- function(x, pars = NA, variable = NULL,
+                                      probs = c(0.025, 0.975),
                                       robust = FALSE, ...) {
   out <- as.matrix(x, pars = pars, variable = variable, ...)
   posterior_summary(out, probs = probs, robust = robust, ...)
 }
 
-# calculate estimates over posterior draws 
+# calculate estimates over posterior draws
 # @param coef coefficient to be applied on the draws (e.g., "mean")
 # @param draws the draws over which to apply coef
 # @param margin see 'apply'
@@ -438,7 +453,7 @@ get_estimate <- function(coef, draws, margin = 2, ...) {
   } else if (coef == "quantile") {
     x <- aperm(x, length(dim(x)):1)
   }
-  x 
+  x
 }
 
 # validate bounds of credible intervals
@@ -462,28 +477,28 @@ validate_ci_bounds <- function(prob, probs = NULL) {
 }
 
 #' Table Creation for Posterior Draws
-#' 
-#' Create a table for unique values of posterior draws. 
-#' This is usually only useful when summarizing predictions 
+#'
+#' Create a table for unique values of posterior draws.
+#' This is usually only useful when summarizing predictions
 #' of ordinal models.
-#' 
-#' @param x A matrix of posterior draws where rows 
-#'   indicate draws and columns indicate parameters. 
+#'
+#' @param x A matrix of posterior draws where rows
+#'   indicate draws and columns indicate parameters.
 #' @param levels Optional values of possible posterior values.
 #'   Defaults to all unique values in \code{x}.
-#' 
-#' @return A matrix where rows indicate parameters 
-#'  and columns indicate the unique values of 
+#'
+#' @return A matrix where rows indicate parameters
+#'  and columns indicate the unique values of
 #'  posterior draws.
-#'  
-#' @examples 
+#'
+#' @examples
 #' \dontrun{
-#' fit <- brm(rating ~ period + carry + treat, 
+#' fit <- brm(rating ~ period + carry + treat,
 #'            data = inhaler, family = cumulative())
 #' pr <- predict(fit, summary = FALSE)
 #' posterior_table(pr)
 #' }
-#'  
+#'
 #' @export
 posterior_table <- function(x, levels = NULL) {
   x <- as.matrix(x)
@@ -497,7 +512,7 @@ posterior_table <- function(x, levels = NULL) {
   if (length(xlevels) != length(levels)) {
     xlevels <- levels
   }
-  out <- lapply(seq_len(ncol(x)), 
+  out <- lapply(seq_len(ncol(x)),
     function(n) table(factor(x[, n], levels = levels))
   )
   out <- do_call(rbind, out)
@@ -508,26 +523,26 @@ posterior_table <- function(x, levels = NULL) {
   out
 }
 
-#' Compute posterior uncertainty intervals 
-#' 
+#' Compute posterior uncertainty intervals
+#'
 #' Compute posterior uncertainty intervals for \code{brmsfit} objects.
-#' 
+#'
 #' @param object An object of class \code{brmsfit}.
-#' @param prob A value between 0 and 1 indicating the desired probability 
+#' @param prob A value between 0 and 1 indicating the desired probability
 #'   to be covered by the uncertainty intervals. The default is 0.95.
 #' @inheritParams as.matrix.brmsfit
 #' @param ... More arguments passed to \code{\link{as.matrix.brmsfit}}.
-#' 
+#'
 #' @return A \code{matrix} with lower and upper interval bounds
 #'   as columns and as many rows as selected variables.
-#'   
-#' @examples 
+#'
+#' @examples
 #' \dontrun{
 #' fit <- brm(count ~ zAge + zBase * Trt,
 #'            data = epilepsy, family = negbinomial())
 #' posterior_interval(fit)
 #' }
-#' 
+#'
 #' @aliases posterior_interval
 #' @method posterior_interval brmsfit
 #' @export
@@ -541,29 +556,29 @@ posterior_interval.brmsfit <- function(
 }
 
 #' Extract Priors of a Bayesian Model Fitted with \pkg{brms}
-#' 
+#'
 #' @aliases prior_summary
-#' 
+#'
 #' @param object An object of class \code{brmsfit}.
-#' @param all Logical; Show all parameters in the model which may have 
+#' @param all Logical; Show all parameters in the model which may have
 #'   priors (\code{TRUE}) or only those with proper priors (\code{FALSE})?
 #' @param ... Further arguments passed to or from other methods.
-#' 
+#'
 #' @return For \code{brmsfit} objects, an object of class \code{brmsprior}.
-#' 
-#' @examples 
+#'
+#' @examples
 #' \dontrun{
-#' fit <- brm(count ~ zAge + zBase * Trt  
-#'              + (1|patient) + (1|obs), 
-#'            data = epilepsy, family = poisson(), 
+#' fit <- brm(count ~ zAge + zBase * Trt
+#'              + (1|patient) + (1|obs),
+#'            data = epilepsy, family = poisson(),
 #'            prior = c(prior(student_t(5,0,10), class = b),
 #'                      prior(cauchy(0,2), class = sd)))
-#'                    
+#'
 #' prior_summary(fit)
 #' prior_summary(fit, all = FALSE)
 #' print(prior_summary(fit, all = FALSE), show_df = FALSE)
 #' }
-#' 
+#'
 #' @method prior_summary brmsfit
 #' @export
 #' @export prior_summary
